@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  2006-2015, University of Amsterdam
+    Copyright (c)  2006-2020, University of Amsterdam
                               VU University Amsterdam
+                              CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -42,6 +43,7 @@
             run_tests/1,                % Run named test-set
             load_test_files/1,          % +Options
             running_tests/0,            % Prints currently running test
+            current_test/5,             % ?Unit,?Test,?Line,?Body,?Options
             test_report/1               % +What
           ]).
 
@@ -49,16 +51,15 @@
 
 Unit testing environment for SWI-Prolog and   SICStus Prolog. For usage,
 please visit http://www.swi-prolog.org/pldoc/package/plunit.
-
-@author         Jan Wielemaker
-@license        GPL+SWI-exception or Artistic 2.0
 */
 
-:- autoload(library(apply),[maplist/3,include/3]).
-:- autoload(library(lists),[member/2,append/2]).
-:- autoload(library(option),[option/3,option/2]).
-:- autoload(library(ordsets),[ord_intersection/3]).
-:- autoload(library(pairs),[group_pairs_by_key/2,pairs_values/2]).
+:- autoload(library(apply), [maplist/3,include/3]).
+:- autoload(library(lists), [member/2,append/2]).
+:- autoload(library(option), [option/3,option/2]).
+:- autoload(library(ordsets), [ord_intersection/3]).
+:- autoload(library(pairs), [group_pairs_by_key/2,pairs_values/2]).
+:- autoload(library(error), [must_be/2]).
+:- autoload(library(thread), [concurrent_forall/2]).
 
 :- meta_predicate valid_options(+, 1).
 
@@ -222,6 +223,11 @@ user:term_expansion((:- thread_local(PI)), (:- dynamic(PI))) :-
 %           If =true= (default =false), cleanup report at the end
 %           of run_tests/1.  Used to improve cooperation with
 %           memory debuggers such as dmalloc.
+%
+%           * concurrent(+Bool)
+%           If =true= (default =false), run all tests in a block
+%           concurrently.
+%
 
 set_test_options(Options) :-
     valid_options(Options, global_test_option),
@@ -237,6 +243,8 @@ global_test_option(sto(Bool)) :-
     must_be(boolean, Bool).
 global_test_option(cleanup(Bool)) :-
     must_be(boolean, Bool).
+global_test_option(concurrent(Bool)) :-
+    must_be(boolean, Bool).
 
 
 %!  loading_tests
@@ -544,6 +552,8 @@ test_set_option(cleanup(X)) :-
     must_be(callable, X).
 test_set_option(sto(V)) :-
     nonvar(V), member(V, [finite_trees, rational_trees]).
+test_set_option(concurrent(V)) :-
+    must_be(boolean, V).
 
 
                  /*******************************
@@ -611,9 +621,15 @@ run_unit(Spec) :-
     ->  info(plunit(blocked(unit(Unit, Reason))))
     ;   setup(Module, unit(Unit), UnitOptions)
     ->  info(plunit(begin(Spec))),
-        forall((Module:'unit test'(Name, Line, Options, Body),
-                matching_test(Name, Tests)),
-               run_test(Unit, Name, Line, Options, Body)),
+        current_test_flag(test_options, GlobalOptions),
+        (   option(concurrent(true), GlobalOptions),
+            option(concurrent(true), UnitOptions, false)
+        ->  concurrent_forall((Module:'unit test'(Name, Line, Options, Body),
+                               matching_test(Name, Tests)),
+                              run_test(Unit, Name, Line, Options, Body))
+        ;   forall((Module:'unit test'(Name, Line, Options, Body),
+                    matching_test(Name, Tests)),
+                   run_test(Unit, Name, Line, Options, Body))),
         info(plunit(end(Spec))),
         (   message_level(silent)
         ->  true
@@ -791,9 +807,6 @@ cleanup_trap_assertions(_).
 :- endif.
 
 
-
-
-
                  /*******************************
                  *         RUNNING A TEST       *
                  *******************************/
@@ -1254,6 +1267,14 @@ running_tests(Running) :-
             ), Running).
 
 
+%!  current_test(?Unit, ?Test, ?Line, ?Body, ?Options)
+%
+%   True when a test with the specified properties is loaded.
+
+current_test(Unit, Test, Line, Body, Options) :-
+    current_unit(Unit, Module, _Supers, _UnitOptions),
+    Module:'unit test'(Test, Line, Options, Body).
+
 %!  check_for_test_errors is semidet.
 %
 %   True if there are no errors, otherwise false.
